/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2015  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const ExtensionUtils = imports.misc.extensionUtils;

const St = imports.gi.St;

const Main = imports.ui.main;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    PanelManager - Battery
 */
const StatusArea = Main.panel.statusArea || Main.panel._statusArea;

const KEY_SHOW_BATTERY = 'show-battery';

const PANELMENU_BUTTON_DEFAULT = 0;
const PANELMENU_BUTTON_SHOW = 1;

const Battery = new Lang.Class({
    Name: 'Battery',

    _init: function() {
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_BATTERY, Lang.bind(this, this._onShowBattery));
        this._connectProxy = StatusArea.battery._proxy.connect('g-properties-changed', Lang.bind(this, this._onShowBattery));
        this._onShowBattery();
    },

    _onShowBattery: function() {
        let visible = StatusArea.battery._proxy.Icon;
        let showBattery = this._settings.get_enum(KEY_SHOW_BATTERY);
        if (showBattery !== PANELMENU_BUTTON_DEFAULT) visible = showBattery === PANELMENU_BUTTON_SHOW;
        if (visible && !StatusArea.battery._proxy.Icon) StatusArea.battery.setIcon('battery-missing-symbolic');
        this._updateVisibility(visible);
    },

    _updateVisibility: function(visible) {
        StatusArea.battery.actor.visible = visible;
        if (StatusArea.battery.mainIcon) StatusArea.battery.mainIcon.visible = visible;
    },

    destroy: function() {
        StatusArea.battery._proxy.disconnect(this._connectProxy);
        this._settings.disconnect(this._connectSettings);
        this._updateVisibility(StatusArea.battery._proxy.Icon);
    }
});

function getExtension() {
    return Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8) ? new Battery() : null;
}
