/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2015  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    PanelManager - Bluetooth
 */
const StatusArea = Main.panel.statusArea || Main.panel._statusArea;

const BLUETOOTH_NO_ADAPTER = -1;

const KEY_SHOW_BLUETOOTH = 'show-bluetooth';

const PANELMENU_BUTTON_DEFAULT = 0;
const PANELMENU_BUTTON_ON = 1;

const Bluetooth = new Lang.Class({
    Name: 'Bluetooth',

    _init: function() {
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_BLUETOOTH, Lang.bind(this, this._onShowBluetooth));
        this._connectBluetoothApplet = [
            StatusArea.bluetooth._applet.connect('devices-changed', Lang.bind(this, this._onShowBluetooth)),
            StatusArea.bluetooth._applet.connect('notify::killswitch-state', Lang.bind(this, this._onShowBluetooth))
        ];
        this._onShowBluetooth();
    },

    _onShowBluetooth: function() {
        let visible = StatusArea.bluetooth._applet.killswitch_state !== BLUETOOTH_NO_ADAPTER;
        let showBluetooth = this._settings.get_enum(KEY_SHOW_BLUETOOTH);
        if (showBluetooth !== PANELMENU_BUTTON_DEFAULT) visible = showBluetooth === PANELMENU_BUTTON_SHOW;
        this._updateVisibility(visible);
    },

    _updateVisibility: function(visible) {
        StatusArea.bluetooth.actor.visible = visible;
    },

    destroy: function() {
        for each(let connection in this._connectBluetoothApplet) {
            StatusArea.bluetooth._applet.disconnect(connection);
        }
        this._settings.disconnect(this._connectSettings);
        this._updateVisibility(StatusArea.bluetooth._applet.killswitch_state !== BLUETOOTH_NO_ADAPTER);
    }
});

function getExtension() {
    return Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8) ? new Bluetooth() : null;
}
