/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2015  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const NMClient = imports.gi.NMClient;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    PanelManager - Network
 */
const StatusArea = Main.panel.statusArea || Main.panel._statusArea;

const KEY_SHOW_NETWORK = 'show-network';

const PANELMENU_BUTTON_DEFAULT = 0;
const PANELMENU_BUTTON_SHOW = 1;

const Network = new Lang.Class({
    Name: 'Network',

    _init: function() {
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_NETWORK, Lang.bind(this, this._onShowNetwork));
        this._client = NMClient.Client.new(null);
        this._connectClient = [
            this._client.connect('notify::manager-running', Lang.bind(this, this._onShowNetwork)),
            this._client.connect('notify::networking-enabled', Lang.bind(this, this._onShowNetwork)),
            this._client.connect('notify::state', Lang.bind(this, this._onShowNetwork))
        ];
        this._remoteSettings = NMClient.RemoteSettings.new(null);
        this._connectRemoteSettings = this._remoteSettings.connect('connections-read', Lang.bind(this, this._onShowNetwork));
        this._onShowNetwork();
    },

    _onShowNetwork: function() {
        let visible = this._client.manager_running;
        let showNetwork = this._settings.get_enum(KEY_SHOW_NETWORK);
        if (showNetwork !== PANELMENU_BUTTON_DEFAULT) visible = showNetwork === PANELMENU_BUTTON_SHOW;
        this._updateVisibility(visible);
    },

    _updateVisibility: function(visible) {
        StatusArea.network.actor.visible = visible;
        if (StatusArea.network.mainIcon) StatusArea.network.mainIcon.visible = visible;
    },

    destroy: function() {
        this._remoteSettings.disconnect(this._connectRemoteSettings);
        for each(let connection in this._connectClient) {
            this._client.disconnect(connection);
        }
        for each(let connection in this._connectSettings) {
            this._settings.disconnect(connection);
        }
        this._updateVisibility(this._client.manager_running);
    }
});

function getExtension() {
    return Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8) ? new Network() : null;
}
