/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2015  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const Shell = imports.gi.Shell;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    PanelManager - ShowDesktop
 */
const KEY_SHOW_SHOWDESKTOP = 'show-showdesktop';

const ShowDesktopIndicator = new Lang.Class({
    Name: 'ShowDesktopIndicator',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(1.0, _("ShowDesktop"), { icon_name: 'video-display-symbolic', dontCreateMenu: true });
        this._windows = null;
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_SHOWDESKTOP, Lang.bind(this, this._onShowShowDesktop));
        this._onShowShowDesktop();
        this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPressEvent));
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._settings.disconnect(this._connectSettings);
        }));
    },

    _onButtonPressEvent: function() {
        this.menu.close(0);
        Main.overview.hide();
        let tracker = Shell.WindowTracker.get_default();
        let workspace = global.screen.get_active_workspace();
        let windows = workspace.list_windows();
        if (this._windows) {
            for (let i = 0; i < windows.length; i++) {
                if (tracker.get_window_app(windows[i])) {
                    let restore = true;
                    for (let j = 0; j < this._windows.length; j++) {
                        if (windows[i] === this._windows[j]) {
                            restore = false;
                            break;
                        }
                    }
                    if (restore) windows[i].unminimize();
                }
            }
            this._windows = null;
        } else {
            this._windows = [];
            for (let i = 0; i < windows.length; i++) {
                if (tracker.get_window_app(windows[i])) {
                    if (!windows[i].minimized) {
                        windows[i].minimize();
                    } else {
                        this._windows.push(windows[i]);
                    }
                }
            }
        }
    },

    _onShowShowDesktop: function() {
        this.actor.visible = this._settings.get_boolean(KEY_SHOW_SHOWDESKTOP);
    }
});

function getExtension() {
    return PanelTools.addToStatusArea('dschaerfShowDesktop', new ShowDesktopIndicator(), 1, 'left');
}
